package org.cebs.corep.excelToXbrl.constants;

/**
 * General constants for the generation of xbrl instance documents.
 */
public interface XbrlConstantsInterface
{
  /** Constant for the overview page of the coreptest excel file with the reference number */
  public static final String OVERVIEW_PAGE = "TestID";

  /** Constant for the directory that contains the taxonomy files */
  public static final String SCHEMA_DIRECTORY = "org/cebs/corep/xbrl/taxonomies/";

  /** Constant for the zip file that contains the taxonomy files */
  public static final String TAXONOMY_ZIP = "Taxonomies1.0.1.zip";

  /** Constants for the xbrl schema namespace */
  public static final String XBRLI_NS = "xbrli";
  public static final String XBRLI_NS_URL = "http://www.xbrl.org/2003/instance";

  /** Constants for the xbrl linkbase namespace */
  public static final String XBRLL_NS = "link";
  public static final String LINKBASE_NS_URL = "http://www.xbrl.org/2003/linkbase";

  /** Constants for the xlink namespace */
  public static final String XLINK_NS = "xlink";
  public static final String XLINK_NS_URL = "http://www.w3.org/1999/xlink";  

  /** Constants for the xbrl unit (EUR) namespace */
  public static final String ISO4217_NS = "iso4217";
  public static final String ISO4217_NS_URL = "http://www.xbrl.org/2003/iso4217";

  /** Constants for the xbrl special data types */
  public static final String DT_NS = "dt";
  public static final String DT_NS_URL = "http://xbrl.c-ebs.org/dt";

  /** Constants for the xbrl dimension schema namespace */
  public static final String XBRLDI_NS = "xbrldi";
  public static final String XBRLDI_NS_URL = "http://xbrl.org/2005/xbrldi";

  /** Constants for the xml schema instance namespace */
  public static final String XSI_NS = "xsi";
  public static final String XSI_NS_URL = "http://www.w3.org/2001/XMLSchema-instance";

  /** Constant for the taxonomy namespace */
  public static final String NS_URL = "http://www.c-ebs.org/eu/fr/esrs/corep/2005-12-31/";
  public static final String NS_DATE = "-2005-12-31";

  public static final String XSD = ".xsd";

  /** Constants for the template namespaces */
  public static final String T_MI_NS = "t-mi";
  public static final String T_MD_NS = "t-md";
  public static final String T_OP_NS = "t-op";
  public static final String T_OD_NS = "t-od";
	public static final String T_OL_NS = "t-ol";
  public static final String T_MT_NS = "t-mt";
  public static final String T_ME_NS = "t-me";
  public static final String T_MF_NS = "t-mf";
  public static final String T_MC_NS = "t-mc";

  /** Constants for the measure namespaces */
  public static final String P_MI_NS = "p-mi";
  public static final String P_MD_NS = "p-md";
  public static final String P_OP_NS = "p-op";
  public static final String P_OD_NS = "p-od";
	public static final String P_OL_NS = "p-ol";
  public static final String P_MT_NS = "p-mt";
  public static final String P_ME_NS = "p-me";
  public static final String P_MF_NS = "p-mf";
  public static final String P_MC_NS = "p-mc";

  /** Constants for the dimension namespaces */
  public static final String D_PM_NS = "d-pm"; //t-mi 
  public static final String D_RI_NS = "d-ri"; // t-mt, t-me
  public static final String D_PC_NS = "d-pc"; // t-mc
	public static final String D_MZ_NS = "d-mz"; // t-mc
  public static final String D_RC_NS = "d-rc"; // t-mf
  public static final String D_BL_NS = "d-bl"; // t-od
  public static final String D_BA_NS = "d-ba"; // t-op
  public static final String D_EV_NS = "d-ev"; // t-od
  public static final String D_TA_NS = "d-ta"; // t-od

  /** Constants for the root element */
  public static final String XBRL = "xbrl";
  public static final String SCHEMAREF = "schemaRef";

  /** Constants for the attributes of the measure elements */
  public static final String CONTEXTREF = "contextRef";
  public static final String DECIMALS = "decimals";
  public static final String UNITREF = "unitRef";

  /** Constants for the attributes of the schemaRef element */
  public static final String ATTRIBUTE_TYPE = "type";
  public static final String ATTRIBUTE_ARCROLE = "arcrole";
  public static final String ATTRIBUTE_HREF = "href";

  /** Constants for the content of the attributes of the schemaRef element */
  public static final String ATTRIBUTE_CONTENT_TYPE = "simple";
  public static final String ATTRIBUTE_CONTENT_ARCROLE = "http://www.w3.org/1999/xlink/properties/linkbase";

  /** Constants for the context elements */
  public static final String CONTEXT = "context";
  public static final String ENTITY = "entity";
  public static final String IDENTIFIER = "identifier";
  public static final String PERIOD = "period";
  public static final String INSTANT = "instant";
  public static final String SCENARIO = "scenario";
  public static final String EXPLICIT_DIM_MEM_REF = "explicitMember";
  public static final String TYPED_DIM_MEM_REF = "typedMember";

  /** Constant for the context elements of tuple based templates */
  public static final String CONTEXT_NO_DIMENSIONS = "cxt_NoDim_";

  /** Constants for the context element of a template that contains dimensions  */
  public static final String CONTEXT_DIMENSIONS = "cxt";
  public static final String CONTEXT_DIMENSION_HREF = "dim_href";

  public static final String DEFAULT_INSTANT = "2005-09-30";

  /** Constants for the unit element */
  public static final String UNIT = "unit";
  public static final String MEASURE = "measure";
  public static final String MEASURE_EUR_CONTENT = "iso4217:EUR";
  public static final String MEASURE_PURE_CONTENT = "xbrli:pure";

  /** Constants for the context and unit attributes */
  public static final String ATTRIBUTE_ID = "id";
  public static final String ATTRIBUTE_SCHEME = "scheme";
  public static final String ATTRIBUTE_SCHEME_CONTENT = "http://www.c-ebs.org/eu/fr/esrs/corep";
  public static final String ATTRIBUTE_EUR_UNIT_CONTENT = "EUR";
  public static final String ATTRIBUTE_PURE_UNIT_CONTENT = "PURE";

  /** Constants for the extracts of the excel sheet names */
  public static final String T_MKR_IM = "MKR IM";
  public static final String T_MKR_DI = "MKR IM Details";
  public static final String T_MKR_SA_TDI = "MKR SA TDI";
  public static final String T_MKR_SA_EQU = "MKR SA EQU";
  public static final String T_MKR_SA_COM = "MKR SA COM";
  public static final String T_MKR_SA_FX = "MKR SA FX";
  public static final String T_OPR = "OPR";
  public static final String T_OPR_DETAILS = "OPR Details";
  public static final String T_OPR_LOSS_DETAILS = "OPR LOSS Details";
}