package org.cebs.corep.excelToXbrl.mapper;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.cebs.corep.xbrl.exceptions.CorepBusinessException;
import org.cebs.corep.xbrl.exceptions.CorepTechnicalException;
import org.cebs.corep.xbrl.log.CorepLogger;
import org.cebs.corep.xbrl.log.LogAction;
import org.cebs.corep.xbrl.util.CorepClientContext;

/**
 * Reads the data out of the excel sheets with  COREP test cases.
 */
public class ExcelSheetReader
{
  private static final String CLASSNAME = "ExcelSheetReader";

  /**
   * The table name and the content of the table of each is read and filled in a HashMap.
   * @param in 			InputStream
   * @return HashMap	
   */
  public static HashMap readWorkbookContent(InputStream in, CorepClientContext cc)
  {
    String method = "readWorkbookContent";
    CorepLogger.log(LogAction.METHOD_BEGIN, CLASSNAME, method, cc);

    // initialize HashMap
    HashMap tableNamesAndRows = new HashMap();
    try
    {
      // Excel file with test cases is read in
      POIFSFileSystem fs = new POIFSFileSystem(in);
      // opens an excel workbook with the data
      HSSFWorkbook workbook = new HSSFWorkbook(fs);

      // ascertains the table name and the content of each table
      getTableContent(workbook, tableNamesAndRows);

      CorepLogger.log(LogAction.METHOD_END, CLASSNAME, method, cc);

      return tableNamesAndRows;
    }
    catch (IOException e)
    {
      CorepLogger.log(LogAction.COREP_XBRL_TECHNICAL_ERROR, CLASSNAME, method, null, "File not readable.", e, cc);
      return null;
    }
    finally
    {
      try
      {
        in.close();
      }
      catch (IOException e)
      {
        CorepLogger.log(LogAction.COREP_XBRL_TECHNICAL_ERROR, CLASSNAME, method, null, "FileInputStream cannot be closed.", e, cc);
      }
    }
  }
  /**
   * This method ascertains the content of each sheet out of a excel workbook
   * and saves it in a HashMap.
   * @param workbook				HSSFWorkbook
   * @param tableNamesAndRows		HashMap
   */
  private static void getTableContent(HSSFWorkbook workbook, HashMap tableNamesAndRows)
  {
    // name of the excel sheet
    String sheetName = null;
    // initialize excel row
    HSSFRow row = null;
    // list for the row content
    ArrayList rows = null;

    // detects the names of the excel sheets
    for (int i = 0; i < workbook.getNumberOfSheets(); i++)
    {
      // new list for every sheet
      rows = new ArrayList();
      // detects the table name of number of the sheet
      sheetName = workbook.getSheetName(i);

      // opens the sheet
      HSSFSheet sheet = workbook.getSheet(sheetName);

      // reads the table row per row
      Iterator iterator = sheet.rowIterator();
      while (iterator.hasNext())
      {
        row = (HSSFRow)iterator.next();
        // adds the row to the rows ArrayList for every table
        rows.add(row);
      }
      // the HashMap is filled with the data of the sheet
      tableNamesAndRows.put(sheetName, rows);
    }
  }

  /**
   * This method reads in the content of an uploaded file into a byte array.
   * @param fileContent		InputStream
   * @param cc				CorepClientContext
   */
  public static InputStream upload(InputStream fileContent, CorepClientContext cc)
  {
    String method = "upload";
    CorepLogger.log(LogAction.METHOD_BEGIN, CLASSNAME, method, cc);

    // writes the file into the ram 
    ByteArrayOutputStream memBuf = new ByteArrayOutputStream();
    int nextByteEncodedAsInt = -1;
    try
    {
      while ((nextByteEncodedAsInt = fileContent.read()) != -1)
        memBuf.write(nextByteEncodedAsInt);
    }
    catch (IOException e)
    {
      CorepLogger.log(LogAction.COREP_XBRL_TECHNICAL_ERROR, CLASSNAME, method, null, "I/O error while uploading the file.", e, cc);
      throw new CorepTechnicalException(e);
    }

    // converts the byte array into an inputStream object
    InputStream in = new ByteArrayInputStream(memBuf.toByteArray());

    CorepLogger.log(LogAction.METHOD_END, CLASSNAME, method, cc);

    return in;
  }

}
