package org.cebs.corep.excelToXbrl.mapper;

import java.util.ArrayList;
import java.util.Properties;

import org.apache.poi.hssf.usermodel.HSSFRow;
import org.cebs.corep.excelToXbrl.constants.MKR_SA_ConstantsInterface;
import org.cebs.corep.excelToXbrl.constants.XbrlConstantsInterface;
import org.jdom.Element;
import org.jdom.Namespace;

/**
 * This class creates a partial xml jdom tree out of a list of HSSFRow objects 
 * for the templates MKR_SA.
 */
public class TemplatesMKRSAtoXbrlMapper extends RowsToXbrlMapper implements XbrlConstantsInterface, MKR_SA_ConstantsInterface
{
  private static final String CLASSNAME = "TemplatesMKRSAtoXbrlMapper";

  /**
   * This class creates a partial xml jdom tree out of a list of HSSFRow objects 
   * for the template MKR_SA_TDI.
   * @param rows
   * @param parentElement
   * @param referenceNumber
   */
  public void transform_MKR_SA_TDI(ArrayList rows, Element parentElement, String referenceNumber, int sheetsCounter, String instantDate)
  {
    // get namespace of the measure taxonomy
    Namespace nsMeasure = Namespace.getNamespace(P_MT_NS, getNamespaceURL(P_MT_NS));

    int cxtCounter = 1; // Counter, die die Context-ID hochzhlt
    String context = CONTEXT_DIMENSIONS + "_" + T_MT_NS + "_" + sheetsCounter + "_"; // in diesem Fall cxt_t-s1_ (gibt zusammen mit dem counter die komplette Context-ID)
    // Nachtrag: sheetsCounter ist noch hinzugekommen
    int col = COLUMN_BEGIN_MKR_SA_TDI;

    // Mapping von Rows zu Dimensions holen
    Properties props = mapDimension_TD_ToRow(context);

    // iteriere ber die einzelnen Zeilen; jede Zeile gehrt immer zur selben Dimension,
    // daher muss am Ende der Context-Zhler hochgezhlt werden (da neue Dimension und damit auch neuer Context)
    for (int i = ROWS_BEGIN_MKR_SA_TDI; i < rows.size(); i++)
    {
      HSSFRow row = (HSSFRow)rows.get(i); // hole die aktuelle Zeile

      createMonetaryElement(parentElement, getMonetaryValue(row, col + 0), ALLPOSITIONSLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 1), ALLPOSITIONSSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 2), REDUCTIONEFFECTFORUNDERWRITINGPOSITIONS, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 3), NETPOSITIONSLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 4), NETPOSITIONSSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 5), TOLONGNETPOSITIONS, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 6), TOSHORTNETPOSITIONS, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 7), NETPOSITIONSSUBJECTTOCAPITALCHARGE, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 9), CAPITALREQUIREMENTS, nsMeasure, context + cxtCounter);

      if (checkRowContent(row, col, new int[] {13}))
      {
        // creates a dimensional context element
        Element contextElement = createDimensionContext(row.getRowNum(), cxtCounter, props, referenceNumber, instantDate);

        // adds the additial dimension to the scenario element of the context
        Element currencyDimension = new Element(CURRENCY, Namespace.getNamespace(T_MT_NS, getNamespaceURL(T_MT_NS)));
        // gets the commodity value
        currencyDimension.addContent(getCellValue((HSSFRow)rows.get(1), 4));

        Element scenario = contextElement.getChild(SCENARIO, Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL));
        scenario.addContent(createTypedDimensionElement(currencyDimension));

        // adds the context element to its parent
        parentElement.addContent(contextElement);
        // raises the context counter
        cxtCounter++;
      }
    }
  }

  /**
   * This method creates a properties object that assigns a dimension element name to a
   * row number.
   * @return 
   */
  private Properties mapDimension_TD_ToRow(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, DIMENSIONNAMETRADEDDEBTINSTRUMENTSTRADINGBOOK);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 1), D_RI_NS + ":" + TOTAL);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 2), D_RI_NS + ":" + GENERALRISKMATURITYBASEDAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 3), D_RI_NS + ":" + GENERALRISKMATURITYBASEDAPPROACHZONE1);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 4), D_RI_NS + ":" + BETWEEN0AND1MONTH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 5), D_RI_NS + ":" + BETWEEN1AND3MONTH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 6), D_RI_NS + ":" + BETWEEN3AND6MONTH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 7), D_RI_NS + ":" + BETWEEN6AND12MONTH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 8), D_RI_NS + ":" + GENERALRISKMATURITYBASEDAPPROACHZONE2);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 9), D_RI_NS + ":" + BETWEEN1AND2YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 10), D_RI_NS + ":" + BETWEEN2AND3YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 11), D_RI_NS + ":" + BETWEEN3AND4YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 12), D_RI_NS + ":" + GENERALRISKMATURITYBASEDAPPROACHZONE3);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 13), D_RI_NS + ":" + BETWEEN4AND5YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 14), D_RI_NS + ":" + BETWEEN5AND7YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 15), D_RI_NS + ":" + BETWEEN7AND10YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 16), D_RI_NS + ":" + BETWEEN10AND15YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 17), D_RI_NS + ":" + BETWEEN15AND20YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 18), D_RI_NS + ":" + GREATER20YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 19), D_RI_NS + ":" + BETWEEN12AND20YEARS3);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 20), D_RI_NS + ":" + GREATER20YEARS3);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 21), D_RI_NS + ":" + MATCHEDWEIGTHEDPOSITIONALLMATURITYBANDS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 22), D_RI_NS + ":" + MATCHEDWEIGHTEDPOSITIONZONE1);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 23), D_RI_NS + ":" + MATCHEDWEIGHTEDPOSITIONZONE2);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 24), D_RI_NS + ":" + MATCHEDWEIGHTEDPOSITIONZONE3);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 25), D_RI_NS + ":" + MATCHEDWEIGTHEDPOSITIONBETWEENZONE1ANDZONE2);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 26), D_RI_NS + ":" + MATCHEDWEIGTHEDPOSITIONBETWEENZONE2ANDZONE3);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 27), D_RI_NS + ":" + MATCHEDWEIGHTEDPOSITIONBETWEENZONE1ANDZONE3);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 28), D_RI_NS + ":" + RESIDUALUNMATCHEDPOSITIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 29), D_RI_NS + ":" + GENERALRISKDURATIONBASEDAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 30), D_RI_NS + ":" + GENERALRISKDURATIONBASEDAPPROACHZONE1);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 31), D_RI_NS + ":" + GENERALRISKDURATIONBASEDAPPROACHZONE2);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 32), D_RI_NS + ":" + GENERALRISKDURATIONBASEDAPPROACHZONE3);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 33), D_RI_NS + ":" + MATCHEDDURATIONWEIGHTEDPOSITIONALLZONES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 34), D_RI_NS + ":" + MATCHEDDURATIONWEIGHTEDPOSITIONBETWEENZONE1ANDZONE2);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 35), D_RI_NS + ":" + MATCHEDDURATIONWEIGHTEDPOSITIONBETWEENZONE2ANDZONE3);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 36), D_RI_NS + ":" + MATCHEDDURATIONWEIGHTEDPOSITIONBETWEENZONE1ANDZONE3);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 37), D_RI_NS + ":" + RESIDUALUNMATCHEDDURATIONWEIGHTEDPOSITIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 38), D_RI_NS + ":" + SPECIFICRISK);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 39), D_RI_NS + ":" + DEBTSECURITIESRSAIRBAPPROACHES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 40), D_RI_NS + ":" + DEBTSECURITIESRSAAPPROACHESOTHERS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 41), D_RI_NS + ":" + RESIDUALTERMS6MONTHS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 42), D_RI_NS + ":" + RESIDUALTERMS6AND24MONTHS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 43), D_RI_NS + ":" + RESIDUALTERMS24MONTHS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 44), D_RI_NS + ":" + DEBTSECURITIESUNDERTHIRDCATEGORYTABLE1);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 45), D_RI_NS + ":" + DEBTSECURITIESUNDERFOURTHCATEGORYTABLE1);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 46), D_RI_NS + ":" + SECURITISATIONEXPOSURESSUBJECT1250PERCENTRISKWEIGHTINGORDEDUCTIONUNRATEDLIQUIDITYFACILITIES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 47), D_RI_NS + ":" + PARTICULARAPPROACHPOSITIONRISKCIUS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 48), D_RI_NS + ":" + MARGINBASEDAPPROACHEXCHANGETRADEDFUTURESOPTIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 49), D_RI_NS + ":" + MARGINBASEDAPPROACHEXCHANGETRADEDOTCFUTURESOPTIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 50), D_RI_NS + ":" + OTHERNONDELTARISKOPTIONS);
    return props;
  }

  /**
   * This class creates a partial xml jdom tree out of a list of HSSFRow objects 
   * for the template MKR_SA_EQU.
   * @param rows
   * @param parentElement
   * @param referenceNumber
   */
  public void transform_MKR_SA_EQU(ArrayList rows, Element parentElement, String referenceNumber, int sheetsCounter, String instantDate)
  {
    // get namespace of the measure taxonomy
    Namespace nsMeasure = Namespace.getNamespace(P_ME_NS, getNamespaceURL(P_ME_NS));

    int cxtCounter = 1; // Counter, die die Context-ID hochzhlt
    String context = CONTEXT_DIMENSIONS + "_" + T_ME_NS + "_" + sheetsCounter + "_"; // in diesem Fall cxt_t-s1_ (gibt zusammen mit dem counter die komplette Context-ID)
    // Nachtrag: sheetsCounter ist noch hinzugekommen
    int col = COLUMN_BEGIN_MKR_SA_EQU;

    // Mapping von Rows zu Dimensions holen
    Properties props = mapDimension_EB_ToRow(context);

    // iteriere ber die einzelnen Zeilen; jede Zeile gehrt immer zur selben Dimension,
    // daher muss am Ende der Context-Zhler hochgezhlt werden (da neue Dimension und damit auch neuer Context)
    for (int i = ROWS_BEGIN_MKR_SA_EQU; i < rows.size(); i++)
    {
      HSSFRow row = (HSSFRow)rows.get(i); // hole die aktuelle Zeile

      createMonetaryElement(parentElement, getMonetaryValue(row, col + 0), ALLPOSITIONSLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 1), ALLPOSITIONSSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 2), REDUTIONEFFECTUNDERWRITINGPOSITIONS, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 3), NETPOSITIONSLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 4), NETPOSITIONSSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 5), NETPOSITIONSSUBJECTCAPITALCHARGE, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 7), CAPITALREQUIREMENTS, nsMeasure, context + cxtCounter);

      if (checkRowContent(row, col))
      {
        // creates a dimensional context element
        Element contextElement = createDimensionContext(row.getRowNum(), cxtCounter, props, referenceNumber, instantDate);

        // adds the additial dimension to the scenario element of the context
        Element nationalMarketDimension = new Element(NATIONALMARKET, Namespace.getNamespace(T_ME_NS, getNamespaceURL(T_ME_NS)));
        // gets the national market value
        nationalMarketDimension.addContent(getCellValue((HSSFRow)rows.get(1), 3));

        Element scenario = contextElement.getChild(SCENARIO, Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL));
        scenario.addContent(createTypedDimensionElement(nationalMarketDimension));

        // adds the context element to its parent
        parentElement.addContent(contextElement);
        // raises the context counter
        cxtCounter++;
      }
    }
  }

  /**
   * This method creates a properties object that assigns a dimension element name to a
   * row number.
   * @return 
   */
  private Properties mapDimension_EB_ToRow(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, DIMENSIONNAMEEQUITITIESTRADINGBOOK);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 1), D_RI_NS + ":" + TOTAL);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 2), D_RI_NS + ":" + GENERALRISK);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 3), D_RI_NS + ":" + EXCHANGETRADEDSTOCKINDEXFUTURESBROADLYDIVERSIFIEDSUBJECTPARTICULARAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 4), D_RI_NS + ":" + OTHEREQUITIESTHANEXCHANGETRADEDSTOCKINDEXFUTURESBROADLYDIVERSIFIED);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 5), D_RI_NS + ":" + SPECIFICRISK);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 6), D_RI_NS + ":" + HIGHQUALITYLIQUIDDIVERSIFIEDPORTFOLIOSSUBJECTLOWERCAPITALREQUIREMENTS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 7), D_RI_NS + ":" + OTHEREQUITIESTHANHIGHQUALITYLIQUIDDIVERSIFIEDPORTFOLIOS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 8), D_RI_NS + ":" + PARTICULARAPPROACHPOSITIONRISKCIUS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 9), D_RI_NS + ":" + MARGINBASEDAPPROACHEXCHANGETRADEDFUTURESOPTIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 10), D_RI_NS + ":" + MARGINBASEDAPPROACHEXCHANGETRADEDOTCFUTURESOPTIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_TDI + 11), D_RI_NS + ":" + OTHERNONDELTARISKOPTIONS);
    return props;
  }

  /**
   * This class creates a partial xml jdom tree out of a list of HSSFRow objects 
   * for the template MKR_SA_COM.
   * @param rows
   * @param parentElement
   * @param referenceNumber
   */
  public void transform_MKR_SA_COM(ArrayList rows, Element parentElement, String referenceNumber, int sheetsCounter, String instantDate)
  {
    // get namespace of the measure taxonomy
    Namespace nsMeasure = Namespace.getNamespace(P_MC_NS, getNamespaceURL(P_MC_NS));

    int cxtCounter = 1; // Counter, die die Context-ID hochzhlt
    String context = CONTEXT_DIMENSIONS + "_" + T_MC_NS + "_" + sheetsCounter + "_"; // in diesem Fall cxt_t-s1_ (gibt zusammen mit dem counter die komplette Context-ID)
    // Nachtrag: sheetsCounter ist noch hinzugekommen
    int col = COLUMN_BEGIN_MKR_SA_COM;

    // Mapping von Rows zu Dimensions holen
    Properties props = mapDimension_PC_ToRow(context);
    Properties propsMZ = mapDimensionMaturityZone(context);

    // iteriere ber die einzelnen Zeilen; jede Zeile gehrt immer zur selben Dimension,
    // daher muss am Ende der Context-Zhler hochgezhlt werden (da neue Dimension und damit auch neuer Context)
    for (int i = ROWS_BEGIN_MKR_SA_COM; i < rows.size(); i++)
    {
      HSSFRow row = (HSSFRow)rows.get(i); // hole die aktuelle Zeile
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 1), ALLPOSITIONSLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 2), ALLPOSITIONSSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 3), POSITIONSPURELYSTOCKFINANCINGLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 4), POSITIONSPURELYSTOCKFINANCINGSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 5), NETPOSITIONSLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 6), NETPOSITIONSSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 7), POSITIONSSUBJECTCAPITALCHARGE, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 9), CAPITALREQUIREMENTS, nsMeasure, context + cxtCounter);

      if (checkRowContent(row, col, new int[] {13}))
      {
        Element contextElement = null;

        contextElement = createDimensionContext(row.getRowNum() - 1, cxtCounter, props, referenceNumber, instantDate);

        // creates a second dimension for the business lines
        createFurtherExplicitDimension(propsMZ, new Integer(row.getRowNum() - 1), contextElement);

        // adds the additial typed dimension to the scenario element of the context
        Element scenario = contextElement.getChild(SCENARIO, Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL));

        Element commodity = new Element(COMMODITY, Namespace.getNamespace(T_MC_NS, getNamespaceURL(T_MC_NS)));
        // gets the commodity value
        commodity.addContent(getCellValue((HSSFRow)rows.get(1), 3));
        scenario.addContent(createTypedDimensionElement(commodity));

        // adds the context element to its parent
        parentElement.addContent(contextElement);
        // raises the context counter
        cxtCounter++;

      }
    }
  }

  /**
   * This method creates a properties object that assigns a dimension element name to a
   * column number.
   * @return
   */
  private Properties mapDimensionMaturityZone(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, DIMENSIONNAMEMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 3), D_MZ_NS + ":" + MATURIYZONELESSEQUAL1YEAR);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 4), D_MZ_NS + ":" + MATURITYZONE1YEARBETWEEN0AND1MONTH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 5), D_MZ_NS + ":" + MATURITYZONE1YEARBETWEEN1AND3MONTHS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 6), D_MZ_NS + ":" + MATURITYZONE1YEARBETWEEN3AND6MONTHS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 7), D_MZ_NS + ":" + MATURITYZONE1YEARBETWEEN6AND12MONTHS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 8), D_MZ_NS + ":" + MATURITYZONEBETWEEN1AND3YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 9), D_MZ_NS + ":" + MATURITYZONEBETWEEN1AND3YEARSBETWEEN1AND2YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 10), D_MZ_NS + ":" + MATURITYZONEBETWEEN1AND3YEARSBETWEEN2AND3YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 11), D_MZ_NS + ":" + MATURIYZONEGREATERTHAN3YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 12), D_MZ_NS + ":" + MATCHEDLONGANDSHORTPOSITIONSWITHINMATURITYBAND);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 13), D_MZ_NS + ":" + MATCHEDPOSITIONSBETWEENTWOMATURITYBANDS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 14), D_MZ_NS + ":" + RESIDUALUNMATCHEDPOSITIONS2);

    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 16), D_MZ_NS + ":" + MATURIYZONELESSEQUAL1YEAR);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 17), D_MZ_NS + ":" + MATURITYZONE1YEARBETWEEN0AND1MONTH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 18), D_MZ_NS + ":" + MATURITYZONE1YEARBETWEEN1AND3MONTHS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 19), D_MZ_NS + ":" + MATURITYZONE1YEARBETWEEN3AND6MONTHS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 20), D_MZ_NS + ":" + MATURITYZONE1YEARBETWEEN6AND12MONTHS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 21), D_MZ_NS + ":" + MATURITYZONEBETWEEN1AND3YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 22), D_MZ_NS + ":" + MATURITYZONEBETWEEN1AND3YEARSBETWEEN1AND2YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 23), D_MZ_NS + ":" + MATURITYZONEBETWEEN1AND3YEARSBETWEEN2AND3YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 24), D_MZ_NS + ":" + MATURIYZONEGREATERTHAN3YEARS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 25), D_MZ_NS + ":" + MATCHEDLONGANDSHORTPOSITIONSWITHINMATURITYBAND);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 26), D_MZ_NS + ":" + MATCHEDPOSITIONSBETWEENTWOMATURITYBANDS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 27), D_MZ_NS + ":" + RESIDUALUNMATCHEDPOSITIONS2);
    return props;
  }

  /**
   * This method creates a properties object that assigns a dimension element name to a
   * row number.
   * @return 
   */
  private Properties mapDimension_PC_ToRow(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, DIMENSIONNAMEPOSITIONSCOMMODITIES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 1), D_PC_NS + ":" + TOTALPOSITIONSCOMMODITIES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 2), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 3), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 4), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 5), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 6), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 7), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 8), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 9), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 10), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 11), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 12), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 13), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 14), D_PC_NS + ":" + MATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 15), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 16), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 17), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 18), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 19), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 20), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 21), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 22), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 23), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 24), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 25), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 26), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 27), D_PC_NS + ":" + EXTENDEDMATURITYLADDERAPPROACH);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 28), D_PC_NS + ":" + SIMPLIFIEDAPPROACHALLPOSITIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 29), D_PC_NS + ":" + NETPOSITIONS2);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 30), D_PC_NS + ":" + GROSSPOSITIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 31), D_PC_NS + ":" + MARGINBASEDAPPROACHEXCHANGETRADEDFUTURESANDOPTIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 32), D_PC_NS + ":" + MARGINBASEDAPPROACHOTCFUTURESANDOPTIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 33), D_PC_NS + ":" + OTHERNONDELTARISKSCOMMODITYOPTIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_COM + 34), D_PC_NS + ":" + RISKSHORTAGELIQUIDITY);
    return props;
  }

  /**
   * This class creates a partial xml jdom tree out of a list of HSSFRow objects 
   * for the template MKR_SA_FX.
   * @param rows
   * @param parentElement
   * @param referenceNumber
   */
  public void transform_MKR_SA_FX(ArrayList rows, Element parentElement, String referenceNumber, int sheetsCounter, String instantDate)
  {
    // get namespace of the measure taxonomy
    Namespace nsMeasure = Namespace.getNamespace(P_MF_NS, getNamespaceURL(P_MF_NS));

    int cxtCounter = 1; // Counter, die die Context-ID hochzhlt
    String context = CONTEXT_DIMENSIONS + "_" + T_MF_NS + "_" + sheetsCounter + "_"; // in diesem Fall cxt_t-s1_ (gibt zusammen mit dem counter die komplette Context-ID)
    // Nachtrag: sheetsCounter ist noch hinzugekommen
    int col = COLUMN_BEGIN_MKR_SA_FX;

    // Mapping von Rows zu Dimensions holen
    Properties props = mapDimension_PR_ToRow(context);

    // iteriere ber die einzelnen Zeilen; jede Zeile gehrt immer zur selben Dimension,
    // daher muss am Ende der Context-Zhler hochgezhlt werden (da neue Dimension und damit auch neuer Context)
    for (int i = ROWS_BEGIN_MKR_SA_FX; i < rows.size(); i++)
    {
      HSSFRow row = (HSSFRow)rows.get(i); // hole die aktuelle Zeile

      createMonetaryElement(parentElement, getMonetaryValue(row, col + 0), ALLPOSITIONSLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 1), ALLPOSITIONSSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 2), HEDGINGPOSITIONSCAPITALRATIOLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 3), HEDGINGPOSITIONSCAPITALRATIOSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 4), NETPOSITIONSLONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 5), NETPOSITIONSSHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 6), POSITIONSSUBJECTCAPITALCHARGELONG, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 7), POSITIONSSUBJECTCAPITALCHARGESHORT, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 8), POSITIONSSUBJECTCAPITALCHARGEMATCHED, nsMeasure, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 12), CAPITALREQUIREMENTS, nsMeasure, context + cxtCounter);

      if (checkRowContent(row, col, new int[] {14, 15, 16}))
      {
        parentElement.addContent(createDimensionContext(row.getRowNum() - 1, cxtCounter, props, referenceNumber, instantDate));

        cxtCounter++;
      }
    }
  }

  /**
   * This method creates a properties object that assigns a dimension element name to a
   * row number.
   * @return 
   */
  private Properties mapDimension_PR_ToRow(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, DIMENSIONNAMEPOSITIONNONREPORTINGCURRENCIES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 1), D_RC_NS + ":" + TOTALPOSITIONSNONREPORTINGCURRENCIES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 2), D_RC_NS + ":" + CURRENCIESSECONDSTAGEEMO);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 3), D_RC_NS + ":" + CURRENCIESSUBJECTINTERGOVERMENTALAGREEMENTS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 4), D_RC_NS + ":" + CURRENCIESCLOSELYCORRELATED);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 5), D_RC_NS + ":" + ALLOTHERCURRENCIES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 6), D_RC_NS + ":" + GOLD);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 7), D_RC_NS + ":" + OTHERNONDELTARISKSCURRENCYOPTIONS);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 9), D_RC_NS + ":" + EURO);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 10), D_RC_NS + ":" + ERM2CURRENCIES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 11), D_RC_NS + ":" + DKK);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 12), D_RC_NS + ":" + EEK);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 13), D_RC_NS + ":" + LTL);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 14), D_RC_NS + ":" + SIT);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 15), D_RC_NS + ":" + GBP);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 16), D_RC_NS + ":" + SEK);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 17), D_RC_NS + ":" + CHF);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 18), D_RC_NS + ":" + OTHEREEECURRENCIES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 19), D_RC_NS + ":" + USD);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 20), D_RC_NS + ":" + CAD);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 21), D_RC_NS + ":" + AUD);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 22), D_RC_NS + ":" + JPY);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 23), D_RC_NS + ":" + OTHERNONEEECURRENCIES);
    props.put(new Integer(ROWS_BEGIN_MKR_SA_FX + 24), D_RC_NS + ":" + CIUSASSEPARATECURRENCIES);
    return props;
  }
}
