package org.cebs.corep.excelToXbrl.mapper;

import java.util.ArrayList;
import java.util.Properties;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.cebs.corep.excelToXbrl.constants.OPR_ConstantsInterface;
import org.cebs.corep.excelToXbrl.constants.XbrlConstantsInterface;
import org.jdom.Element;
import org.jdom.Namespace;

/**
 * This class creates a partial xml jdom tree out of a list of HSSFRow objects 
 * for the templates OPR und OPR LOSS.
 */
public class TemplatesOPRtoXbrlMapper extends RowsToXbrlMapper implements XbrlConstantsInterface, OPR_ConstantsInterface
{
  /**
   * This method transforms the rows of the template OPR in a xbrl representation.
   * 
   * @param rows			ArrayList
   * @param parentElement	Element
   */
  public void transform_OPR(ArrayList rows, Element parentElement, String referenceNumber, int sheetsCounter, String instantDate)
  {
    // get namespace of the measure taxonomy
    Namespace ns = Namespace.getNamespace(P_OP_NS, getNamespaceURL(P_OP_NS));

    String context = CONTEXT_PART + T_OP_NS + "_" + sheetsCounter + "_";

    // creates a properties object that map a dimension element name to a row
    Properties props = mapDimensionBankingActivities(context);
    Properties propsBL = mapDimensionBusinessLines(context);

    int cxtCounter = 1;
    int col = COLUMN_BEGIN_OPR;

    // creates the partial xml tree based on the rows of the template
    for (int i = ROWS_BEGIN_OPR; i < ROWS_END_OPR; i++)
    {
      HSSFRow row = (HSSFRow)rows.get(i);

      createMonetaryElement(parentElement, getMonetaryValue(row, col), YEAR3GI, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 1), YEAR2GI, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 2), LASTYEARGI, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 3), YEAR3LA, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 4), YEAR2LA, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 5), LASTYEARLA, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 6), CAPITALREQUIREMENTS, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 7), OFWHICHALLOCATIONMECHANISM, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 8), CAPITALREQUIREMENTSBEFOREALLEVIATION, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 9), ALLEVIATIONCAPITALREQUIREMENTSCAPTUREDINBUSINESSPRACTICES, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 10), ALLEVIATIONCAPITALREQUIREMENTSRISKTRANSFERMECHANISMS, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 12), OFWHICHDUEINSURANCE, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 13), EXCESSONLIMITCAPITALALLEVIATION, ns, context + cxtCounter);

      // adds the context element for each row
      if (checkRowContent(row, col))
      {
        // creates a dimensional context element for the banking activities dimension
        Element contextElement = createDimensionContext(row.getRowNum(), cxtCounter, props, referenceNumber, instantDate);

        // creates a second dimension for the business lines
        createFurtherExplicitDimension(propsBL, new Integer(row.getRowNum()), contextElement);

        // adds the context element to its parent
        parentElement.addContent(contextElement);

        cxtCounter++;
      }
    }
  }

  /**
   * This method creates a properties object that assigns a dimension element name to a
   * row number.
   * @return
   */
  private Properties mapDimensionBankingActivities(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, getDimensionLink(T_OP_NS, BANKINGACTIVITIESDIMENSION));
    props.put(new Integer(ROWS_BEGIN_OPR), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTTOBIA);
    props.put(new Integer(ROWS_BEGIN_OPR + 1), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTSTAALTERNATIVE);
    props.put(new Integer(ROWS_BEGIN_OPR + 3), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTSTA);
    props.put(new Integer(ROWS_BEGIN_OPR + 4), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTSTA);
    props.put(new Integer(ROWS_BEGIN_OPR + 5), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTSTA);
    props.put(new Integer(ROWS_BEGIN_OPR + 6), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTSTA);
    props.put(new Integer(ROWS_BEGIN_OPR + 7), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTSTA);
    props.put(new Integer(ROWS_BEGIN_OPR + 8), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTSTA);
    props.put(new Integer(ROWS_BEGIN_OPR + 9), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTSTA);
    props.put(new Integer(ROWS_BEGIN_OPR + 10), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTSTA);
    props.put(new Integer(ROWS_BEGIN_OPR + 12), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTASA);
    props.put(new Integer(ROWS_BEGIN_OPR + 13), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTASA);
    props.put(new Integer(ROWS_BEGIN_OPR + 14), D_BA_NS + ":" + TOTALBANKINGACTIVITIESSUBJECTTOAMA);

    return props;
  }

  /**
  	 * This method creates a properties object that assigns a dimension element name to a
  	 * row number.
  	 * @return
  	 */
  private Properties mapDimensionBusinessLines(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, getDimensionLink(T_OP_NS, BUSINESSLINESDIMENSION));
    props.put(new Integer(ROWS_BEGIN_OPR + 3), D_BL_NS + ":" + CORPORATEFINANCE);
    props.put(new Integer(ROWS_BEGIN_OPR + 4), D_BL_NS + ":" + TRADINGANDSALES);
    props.put(new Integer(ROWS_BEGIN_OPR + 5), D_BL_NS + ":" + RETAILBROKERAGE);
    props.put(new Integer(ROWS_BEGIN_OPR + 6), D_BL_NS + ":" + COMMERCIALBANKING);
    props.put(new Integer(ROWS_BEGIN_OPR + 7), D_BL_NS + ":" + RETAILBANKING);
    props.put(new Integer(ROWS_BEGIN_OPR + 8), D_BL_NS + ":" + PAYMENTANDSETTLEMENT);
    props.put(new Integer(ROWS_BEGIN_OPR + 9), D_BL_NS + ":" + AGENCYSERVICES);
    props.put(new Integer(ROWS_BEGIN_OPR + 10), D_BL_NS + ":" + ASSETMANAGEMENT);
    props.put(new Integer(ROWS_BEGIN_OPR + 12), D_BL_NS + ":" + COMMERCIALBANKING);
    props.put(new Integer(ROWS_BEGIN_OPR + 13), D_BL_NS + ":" + RETAILBANKING);

    return props;
  }

  /**
   * This method transforms the rows of the template OPR Details in a xbrl representation.
   * 
   * @param rows			ArrayList
   * @param parentElement	Element
   */
  public void transform_OPR_DETAILS(ArrayList rows, Element parentElement, String referenceNumber, int sheetsCounter, String instantDate)
  {
    // get namespace of the measure taxonomy
    Namespace ns = Namespace.getNamespace(P_OD_NS, getNamespaceURL(P_OD_NS));

    int cxtCounter = 1;
    String context = CONTEXT_PART + T_OD_NS + "_" + sheetsCounter + "_";

    // creates a properties object that map a dimension element name to a column
    Properties props = mapDimensionEventTypes(context);
    //	creates a properties object that map a dimension element name to a row
    Properties propsBL = mapDimensionToRowBusinessLines(context);
    //	creates a properties object that map a dimension element name to a row
    Properties propsMI = mapDimensionMemordandumItems(context);

    int col;

    int rowNum = ROWS_BEGIN_OPR_DETAILS;
    while (rowNum < rows.size())
    {
      // creates the partial xml tree based on the rows of the template
      for (col = COLUMN_BEGIN_OPR_DETAILS; col < COLUMN_END_OPR_DETAILS; col++)
      {
        createMonetaryElement(parentElement, getMonetaryValue((HSSFRow)rows.get(rowNum), col), NUMBEREVENTS, ns, context + cxtCounter);
        createMonetaryElement(parentElement, getMonetaryValue((HSSFRow)rows.get(rowNum + 1), col), TOTALLOSSAMOUNT, ns, context + cxtCounter);
        createMonetaryElement(parentElement, getMonetaryValue((HSSFRow)rows.get(rowNum + 2), col), MAXIMUMSINGLELOSS, ns, context + cxtCounter);
        // adds the context element for each column
        if (StringUtils.isNotBlank(getCellValue((HSSFRow)rows.get(rowNum), col)) || StringUtils.isNotBlank(getCellValue((HSSFRow)rows.get(rowNum + 1), col)) || StringUtils.isNotBlank(getCellValue((HSSFRow)rows.get(rowNum + 2), col)))
        {
          Element contextElement = null;
          if (col < 11)
            contextElement = createDimensionContext(col, cxtCounter, props, referenceNumber, instantDate);
          else
            // creates a second dimension for the business lines
            contextElement = createDimensionContext(col, cxtCounter, propsMI, referenceNumber, instantDate);

          // creates a second dimension for the business lines
          createFurtherExplicitDimension(propsBL, new Integer(rowNum), contextElement);

          // adds the context element to its parent
          parentElement.addContent(contextElement);

          // raises the context counter
          cxtCounter++;
        }
      }
      rowNum = rowNum + 3;
    }
  }

  /**
   * This method creates a properties object that assigns a dimension element name to a
   * column number.
   * @return
   */
  private Properties mapDimensionEventTypes(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, getDimensionLink(T_OD_NS, EVENTTYPEDIMENSION));
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS), D_EV_NS + ":" + INTERNALFRAUD);
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS + 1), D_EV_NS + ":" + EXTERNALFRAUD);
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS + 2), D_EV_NS + ":" + EMPLOYMENTPRACTICESWORKPLACESAFETY);
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS + 3), D_EV_NS + ":" + CLIENTSPRODUCTSBUSINESSPRACTICES);
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS + 4), D_EV_NS + ":" + DAMAGEPHYSICALASSETS);
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS + 5), D_EV_NS + ":" + BUSINESSDISRUPTIONSYSTEMFAILURES);
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS + 6), D_EV_NS + ":" + EXECUTIONDELIVERYPROCESSMANAGEMENT);
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS + 7), D_EV_NS + ":" + TOTAL);
    return props;
  }

  /**
   * This method creates a properties object that assigns a dimension element name to a
   * column number.
   * @return
   */
  private Properties mapDimensionToRowBusinessLines(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, getDimensionLink(T_OD_NS, BUSINESSLINESDIMENSION2));
    props.put(new Integer(ROWS_BEGIN_OPR_DETAILS), D_BL_NS + ":" + CORPORATEFINANCE);
    props.put(new Integer(ROWS_BEGIN_OPR_DETAILS + 3), D_BL_NS + ":" + TRADINGANDSALES);
    props.put(new Integer(ROWS_BEGIN_OPR_DETAILS + 6), D_BL_NS + ":" + RETAILBROKERAGE);
    props.put(new Integer(ROWS_BEGIN_OPR_DETAILS + 9), D_BL_NS + ":" + COMMERCIALBANKING);
    props.put(new Integer(ROWS_BEGIN_OPR_DETAILS + 12), D_BL_NS + ":" + RETAILBANKING);
    props.put(new Integer(ROWS_BEGIN_OPR_DETAILS + 15), D_BL_NS + ":" + PAYMENTANDSETTLEMENT);
    props.put(new Integer(ROWS_BEGIN_OPR_DETAILS + 18), D_BL_NS + ":" + AGENCYSERVICES);
    props.put(new Integer(ROWS_BEGIN_OPR_DETAILS + 21), D_BL_NS + ":" + ASSETMANAGEMENT);
    props.put(new Integer(ROWS_BEGIN_OPR_DETAILS + 24), D_BL_NS + ":" + TOTAL);
    return props;
  }

  /**
  	 * This method creates a properties object that assigns a dimension element name to a
  	 * column number.
  	 * @return
  	 */
  private Properties mapDimensionMemordandumItems(String context)
  {
    Properties props = new Properties();
    props.put(CONTEXT_DIMENSIONS, context);
    props.put(CONTEXT_DIMENSION_HREF, THRESHOLDAPPLIEDINDATACOLLECTIONDIMENSION);
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS + 8), D_TA_NS + ":" + LOWEST);
    props.put(new Integer(COLUMN_BEGIN_OPR_DETAILS + 9), D_TA_NS + ":" + HIGHEST);
    return props;
  }

  /**
   * This method transforms the rows of the template OPR LOSS Details in a xbrl representation.
   * 
   * @param rows			ArrayList
   * @param parentElement	Element
   */
  public void transform_OPR_LOSS(ArrayList rows, Element parentElement, String referenceNumber, int sheetsCounter, String instantDate)
  {
    // get namespace of the measure taxonomy
    Namespace ns = Namespace.getNamespace(P_OL_NS, getNamespaceURL(P_OL_NS));

    int cxtCounter = 1;
    String context = CONTEXT_PART + T_OL_NS + "_" + sheetsCounter + "_";

    int col = COLUMN_BEGIN_OPR;

    // creates the partial xml tree based on the rows of the template
    for (int i = ROWS_BEGIN_OPR_LOSS; i < rows.size(); i++)
    {
      HSSFRow row = (HSSFRow)rows.get(i);

      createMonetaryElement(parentElement, getMonetaryValue(row, col), GROSSLOSSAMOUNT, ns, context + cxtCounter);
			createMonetaryElement(parentElement, getMonetaryValue(row, col + 1), OFWHICHUNREALIZED, ns, context + cxtCounter);  
			createUniversalElement(parentElement, getCellValue(row, col + 2), STATUS, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 3), LOSSALREADYDIRECTLYRECOVERED, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 4), LOSSALREADYRECOVEREDFROMRISKTRANSFERMECHANISMS, ns, context + cxtCounter);
      createMonetaryElement(parentElement, getMonetaryValue(row, col + 5), LOSSPOTENTIALLYTOBERECOVEREDDIRECTLYORFROMRISKTRANSFERMECHANISMS, ns, context + cxtCounter);
			createUniversalElement(parentElement, getCellValue(row, col + 6), RELATEDTOCRORMKR, ns, context + cxtCounter);
			createPercentageElement(parentElement, getCellValue(row, col + 7), CORPORATEFINANCE, ns, context + cxtCounter);
			createPercentageElement(parentElement, getCellValue(row, col + 8), TRADINGSALES, ns, context + cxtCounter);
			createPercentageElement(parentElement, getCellValue(row, col + 9), RETAILBROKERAGE, ns, context + cxtCounter);
			createPercentageElement(parentElement, getCellValue(row, col + 10), COMMERCIALBANKING, ns, context + cxtCounter);
			createPercentageElement(parentElement, getCellValue(row, col + 11), RETAILBANKING, ns, context + cxtCounter);
			createPercentageElement(parentElement, getCellValue(row, col + 12), PAYMENTSETTLEMENT, ns, context + cxtCounter);
			createPercentageElement(parentElement, getCellValue(row, col + 13), AGENCYSERVICES, ns, context + cxtCounter);
			createPercentageElement(parentElement, getCellValue(row, col + 14), ASSETMANAGEMENT, ns, context + cxtCounter);
			createIntegerElement(parentElement, getCellValue(row, col + 15), RISKEVENTTYPE, ns, context + cxtCounter);
			createUniversalElement(parentElement, getDateValue(row, col + 16), OCURRENCE, ns, context + cxtCounter);
			createUniversalElement(parentElement, getDateValue(row, col + 17), RECOGNITION, ns, context + cxtCounter);
			createUniversalElement(parentElement, getDateValue(row, col + 18), FIRSTPAYMENTFROMRISKTRANSFERMECHANISMS, ns, context + cxtCounter);
			createUniversalElement(parentElement, getDateValue(row, col + 19), LATESTPAYMENTFROMRISKTRANSFERMECHANISMS, ns, context + cxtCounter);

      // adds the context element for each row
      if (checkRowContent(row, col))
      {
        // creates a dimensional context element for the banking activities dimension
        Element contextElement = createGeneralContextElement(context + cxtCounter, referenceNumber, instantDate);

        // creates a scenario element 
        Element scenario = new Element(SCENARIO, Namespace.getNamespace(XBRLI_NS, XBRLI_NS_URL));
        //	adds the additial dimension to the scenario element of the context
        Element referenceElement = new Element(INTERNALREFERENCENUMBERDIMENSION, Namespace.getNamespace(T_OL_NS, getNamespaceURL(T_OL_NS)));
        // gets the national market value
				referenceElement.addContent(getCellValue(row, col - 1));

        // add the implicit dimension "reference number"
        scenario.addContent(createTypedDimensionElement(referenceElement));

        // adds the context element to its parent
        contextElement.addContent(scenario);
        parentElement.addContent(contextElement);

        cxtCounter++;
      }
    }
  }
}